@@ -13,13 +13,13 @@ from dldtrainer.utils import plot_utils
 
 if __name__ == "__main__":
     num_epochs = 150
-    num_warn_up = 10
+    num_warn_up = 3
     lr_init = 0.1
     num_steps = 10
     milestones = [20, 30, 40]
     model = models.resnet18(pretrained=False)
     optimizer = optim.SGD(model.parameters(), lr=lr_init)
-    scheduler = "multi-step1"  # cosine,multi-step
+    scheduler = "cosine"  # cosine,multi-step
     lr_scheduler = build_scheduler.get_scheduler(scheduler,
                                                  optimizer=optimizer,
                                                  lr_init=lr_init,
@@ -30,6 +30,7 @@ if __name__ == "__main__":
 
     lr_list = []
     for epoch in range(num_epochs):
+        lr_scheduler.on_epoch_begin(epoch)
         for step in range(num_steps):
             lr_scheduler.step(epoch, step)
         lr = optimizer.param_groups[0]["lr"]
